@@ -53,6 +53,7 @@
 #'   \code{\link[utils]{Stangle}})
 #' @param text a character vector as an alternative way to provide the input
 #'   file
+#' @param quiet whether to suppress the progress bar and messages
 #' @param envir the environment in which the code chunks are to be evaluated
 #'   (can use \code{\link{new.env}()} to guarantee an empty new environment)
 #' @param encoding the encoding of the input file; see \code{\link{file}}
@@ -95,7 +96,7 @@
 #' purl(f)  # tangle R code
 #' purl(f, documentation = 0)  # extract R code only
 #' purl(f, documentation = 2)  # also include documentation
-knit = function(input, output = NULL, tangle = FALSE, text = NULL,
+knit = function(input, output = NULL, tangle = FALSE, text = NULL, quiet = FALSE,
                 envir = parent.frame(), encoding = getOption('encoding')) {
 
   # is input from a file? (or a connection on a file)
@@ -130,7 +131,11 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL,
     ocode = knit_code$get(); on.exit(knit_code$restore(ocode), add = TRUE)
     if (tangle) knit_code$restore() # clean up code before tangling
     optk = opts_knit$get(); on.exit(opts_knit$set(optk), add = TRUE)
-    opts_knit$set(tangle = tangle, encoding = encoding)
+    # use the option KNITR_PROGRESS to control the progress bar
+    opts_knit$set(
+      tangle = tangle, encoding = encoding,
+      progress = opts_knit$get('progress') && getOption('KNITR_PROGRESS', TRUE) && !quiet
+    )
   }
 
   ext = 'unknown'
@@ -138,14 +143,9 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL,
     input.dir = .knitEnv$input.dir; on.exit({.knitEnv$input.dir = input.dir}, add = TRUE)
     .knitEnv$input.dir = dirname(input) # record input dir
     ext = tolower(file_ext(input))
-    if (is.null(output)) output = basename(auto_out_name(input, ext))
+    if (is.null(output) && !child_mode()) output = basename(auto_out_name(input, ext))
     options(tikzMetricsDictionary = tikz_dict(input)) # cache tikz dictionary
-    knit_concord$set(infile = input)
-  }
-  if (concord_mode()) {
-    # 'outfile' from last parent call is my parent
-    if (child_mode()) knit_concord$set(parent = knit_concord$get('outfile'))
-    knit_concord$set(outfile = output)
+    knit_concord$set(infile = input, outfile = output)
   }
 
   encoding = correct_encode(encoding)
@@ -189,7 +189,7 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL,
   }
 
   progress = opts_knit$get('progress')
-  if (in.file) message(ifelse(progress, '\n\n', ''), 'processing file: ', input)
+  if (in.file && !quiet) message(ifelse(progress, '\n\n', ''), 'processing file: ', input)
   res = process_file(text, output)
   res = paste(knit_hooks$get('document')(res), collapse = '\n')
   if (!is.null(output))
@@ -201,13 +201,8 @@ knit = function(input, output = NULL, tangle = FALSE, text = NULL,
   }
 
   if (in.file && is.character(output) && file.exists(output)) {
-    concord_gen(input2, output)  # concordance file
-    if (!child_mode() && concord_mode()) {
-      confile = str_c(file_path_sans_ext(output), '-concordance.tex')
-      cat(.knitEnv$concordance, file = confile)
-      .knitEnv$concordance = NULL # empty concord string
-    }
-    message('output file: ', normalizePath(output), ifelse(progress, '\n', ''))
+    concord_gen(input, output)
+    if (!quiet) message('output file: ', output, ifelse(progress, '\n', ''))
   }
 
   output %n% res
@@ -233,11 +228,10 @@ process_file = function(text, output) {
   tangle = opts_knit$get('tangle')
 
   if (opts_knit$get('progress')) {
-    pb = txtProgressBar(0, n, char = '>', style = 3)
+    pb = txtProgressBar(0, n, char = '.', style = 3)
     on.exit(close(pb), add = TRUE)
   }
   for (i in 1:n) {
-    knit_concord$set(i = i)
     if (opts_knit$get('progress')) {
       setTxtProgressBar(pb, i)
       if (!tangle) cat('\n')  # under tangle mode, only show one progress bar
@@ -254,22 +248,18 @@ process_file = function(text, output) {
         )
       }
     )
-    # output line numbers
-    if (concord_mode()) {
-      # look back and see who is 0, then fill them up
-      idx = which(olines[1:i] == 0L); olines[idx] = line_count(res[idx])
-      knit_concord$set(outlines = olines)
-    }
   }
 
   if (!tangle) res = insert_header(res)  # insert header
+  # output line numbers
+  if (concord_mode()) knit_concord$set(outlines = line_count(res))
   print_knitlog()
 
   res
 }
 
 auto_out_name = function(input, ext = tolower(file_ext(input))) {
-  base = file_path_sans_ext(input)
+  base = sans_ext(input)
   if (opts_knit$get('tangle')) return(str_c(base, '.R'))
   if (ext %in% c('rnw', 'snw')) return(str_c(base, '.tex'))
   if (ext %in% c('rmd', 'rmarkdown', 'rhtml', 'rhtm', 'rtex', 'stex', 'rrst'))
@@ -307,49 +297,29 @@ auto_format = function(ext) {
 #' the result into the main document. It is designed to be used in the chunk
 #' option \code{child} and serves as the alternative to the
 #' \command{SweaveInput} command in Sweave.
-#'
-#' For LaTeX output, the command used to input the child document (usually
-#' \samp{input} or \samp{include}) is from the package option
-#' \code{child.command} (\code{opts_knit$get('child.command')}). For other types
-#' of output, the content of the compiled child document is returned.
-#'
-#' When we call \code{purl()} to extract R code, the code in the child document
-#' is extracted and saved into an R script.
-#'
-#' The path of the child document is relative to the parent document.
 #' @param ... arguments passed to \code{\link{knit}}
 #' @param eval logical: whether to evaluate the child document
-#' @return A character string of the form \samp{\command{child-doc.tex}} or
-#'   \code{source("child-doc.R")}, depending on the argument \code{tangle}
-#'   passed in. When concordance is turned on or the output format is not LaTeX,
-#'   the content of the compiled child document is returned as a character
-#'   string so it can be written back to the main document directly.
+#' @return A character string of the content of the compiled child document is
+#'   returned as a character string so it can be written back to the parent
+#'   document directly.
 #' @references \url{http://yihui.name/knitr/demo/child/}
 #' @note This function is not supposed be called directly like
 #'   \code{\link{knit}()}; instead it must be placed in a parent document to let
 #'   \code{\link{knit}()} call it indirectly.
+#'
+#'   The path of the child document is relative to the parent document.
 #' @export
-#' @examples ## you can write \Sexpr{knit_child('child-doc.Rnw')} in an Rnw file 'main.Rnw' to input child-doc.tex in main.tex
+#' @examples ## you can write \Sexpr{knit_child('child-doc.Rnw')} in an Rnw file 'main.Rnw' to input results from child-doc.Rnw in main.tex
 #'
 #' ## comment out the child doc by \Sexpr{knit_child('child-doc.Rnw', eval = FALSE)}
-#'
-#' ## use \include: opts_knit$set(child.command = 'include')
 knit_child = function(..., eval = TRUE) {
   if (!eval) return('')
   child = child_mode()
   opts_knit$set(child = TRUE) # yes, in child mode now
   on.exit(opts_knit$set(child = child)) # restore child status
-  path = knit(..., tangle = opts_knit$get('tangle'),
-              encoding = opts_knit$get('encoding') %n% getOption('encoding'))
-  if (is.null(path)) return() # the input document is empty
-  if (opts_knit$get('tangle')) {
-    str_c('\n', 'source("', path, '")')
-  } else if (concord_mode() || !out_format('latex')) {
-    on.exit(unlink(path)) # child output file is temporary
-    str_c(readLines(path), collapse = '\n')
-  } else {
-    str_c('\n\\', opts_knit$get('child.command'), '{', path, '}')
-  }
+  res = knit(..., tangle = opts_knit$get('tangle'),
+             encoding = opts_knit$get('encoding') %n% getOption('encoding'))
+  paste(c('', res), collapse = '\n')
 }
 
 knit_log = new_defaults()  # knitr log for errors, warnings and messages
