@@ -3,11 +3,10 @@
 #' @author Ramnath Vaidyanathan
 set_theme = function(theme) {
   header = if (is.list(theme)) theme else theme_to_header(theme)
-  highlight = paste(header$highlight, collapse = "\n")
   if(out_format('latex')) {
   	opts_chunk$set(background = header$background)
   }
-  set_header(highlight = highlight)
+  set_header(highlight = header$highlight)
   ## par(col = theme$foreground)
 }
 get_theme = function(theme = NULL) {
@@ -71,7 +70,7 @@ theme_to_header_latex = function(theme) {
 
   ## write latex highlight header
   fgheader = color_def(foreground, "fgcolor")
-  highlight = c(fgheader, styler_assistant_latex(css_out[-1]))
+  highlight = paste(c(fgheader, styler_assistant_latex(css_out[-1])), collapse = '\n')
   list(highlight = highlight, background = background, foreground = foreground)
 }
 
@@ -87,6 +86,6 @@ theme_to_header_html = function(theme){
   css_knitr = readLines(system.file('misc', 'knitr.css', package = 'knitr'))
   css_knitr[-2] = sub('^(\\s+background-color:\\s+)(.*)$', sprintf('\\1%s;', bgcolor), css_knitr[-2])
   css = c(css_knitr, sprintf('.source {\n  color: %s;\n}', fgcolor), readLines(css_file))
-  list(highlight = css)
+  list(highlight = paste(css, collapse = '\n'))
 }
 
