@@ -1,8 +1,10 @@
 .onLoad = function(lib, pkg) {
-  if (R.version$`svn rev` >= 61843 && getRversion() >= '3.0.0')
-    tools::vignetteEngine('knitr', weave = vweave, tangle = vtangle)
-  # use the option KNITR_PROGRESS to control the progress bar
-  opts_knit$set(progress = getOption('KNITR_PROGRESS', TRUE))
+  if (getRversion() >= '3.0.0') {
+    tools::vignetteEngine(
+      'knitr', weave = vweave, tangle = vtangle,
+      pattern = c('[.][rRsS](nw|tex)$', '[.][Rr](md|html)$'), package = pkg
+    )
+  }
   # add /usr/texbin to PATH for Mac OS
   if (Sys.info()['sysname'] != 'Darwin') return()
   if (!file.exists(texbin <- '/usr/texbin')) return()
