@@ -3,9 +3,10 @@ options(device.ask.default = FALSE)
 
 shinyServer(function(input, output) {
 
-  output$nbOut = reactive(function() {
+  output$nbOut = reactive({
     src = input$nbSrc
     library(knitr)
+    opts_knit$set(progress = FALSE)
     if (length(src) == 0L || src == '')
       return('Nothing to show yet...')
     on.exit(unlink('figure/', recursive = TRUE)) # do not need the figure dir
